/*
 * Cantata
 *
 * Copyright (c) 2017 Craig Drummond <craig.p.drummond@gmail.com>
 *
 * ----
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "statsdb.h"
#include "scrobbling/pausabletimer.h"
#include "config.h"
#include "mpd-interface/mpdconnection.h"
#include "support/globalstatic.h"
#include "support/utils.h"
#include "support/configuration.h"
#include <QUrl>
#include <QStringList>
#include <QTimer>
#include <QFile>
#include <QDir>

#include <QDebug>
static bool debugIsEnabled=false;
static bool fakeScrobbling=false;
#define DBUG if (debugIsEnabled) qWarning() << metaObject()->className() << __FUNCTION__
#define DBUG_CLASS(C) if (debugIsEnabled) qWarning() << C << __FUNCTION__
void StatsDb::enableDebug()
{
    debugIsEnabled=true;
}

static const QLatin1String constSettingsGroup("Stats");
static const int constNowPlayingInterval=5000;

GLOBAL_STATIC(StatsDb, instance)

StatsDb::StatsDb()
    : QObject(0)
    , enabled(false)
    , commitedCurrent(false)
{
    commitTimer = new PausableTimer();
    commitTimer->setInterval(9000000);
    commitTimer->setSingleShot(true);
    connect(commitTimer, SIGNAL(timeout()), this, SLOT(commitCurrent()));
    loadSettings();
    connect(MPDConnection::self(), SIGNAL(statusUpdated(MPDStatusValues)), this, SLOT(mpdStatusUpdated(MPDStatusValues)));
    connect(MPDConnection::self(), SIGNAL(currentSongUpdated(Song)), this, SLOT(setSong(Song)));
}

StatsDb::~StatsDb()
{
    DBUG;
    commitTimer->stop();
}

void StatsDb::setActive()
{
    if (enabled) {
        connect(MPDStatus::self(), SIGNAL(updated()), this, SLOT(mpdStateUpdated()));
    } else {
        disconnect(MPDStatus::self(), SIGNAL(updated()), this, SLOT(mpdStateUpdated()));
    }

    if (enabled && !inactiveSong.isEmpty()) {
        commitedCurrent=false;
        setSong(inactiveSong);
        inactiveSong.clear();
    }
}

void StatsDb::loadSettings()
{
    Configuration cfg(constSettingsGroup);

    enabled=cfg.get("enabled", enabled);
    setActive();
}

void StatsDb::calcCommitTime()
{
    int elapsed=MPDStatus::self()->timeElapsed()*1000;
    if (elapsed<0) {
        elapsed=0;
    }
    int timeout=qMin(currentSong.time/2, 60)*1000; // Scrobble at 1/2 way point or 1 min - whichever comes first!
    DBUG << "timeout" << timeout << elapsed;
    if (timeout>elapsed) {
        timeout-=elapsed;
    } else {
        timeout=100;
    }
    commitTimer->setInterval(timeout);
}

void StatsDb::setSong(const Song &s)
{
    DBUG << enabled << s.isStandardStream() << s.time << s.file << s.title << s.artist << s.album << s.albumartist << s.blank;
    if (!enabled) {
        inactiveSong=s;
        return;
    }

    inactiveSong.clear();
    if (currentSong.artist != s.artist || currentSong.title!=s.title || currentSong.album!=s.album) {
        currentSong=s;
        calcCommitTime();
        if (MPDState_Playing==MPDStatus::self()->state()) {
            mpdStateUpdated(true);
        }
    }
}

void StatsDb::commitCurrent()
{
    if (!commitedCurrent) {
        DBUG << currentSong.artist << currentSong.title;
        currentSong.clear();
        commitedCurrent = true;
    }
}

void StatsDb::mpdStateUpdated(bool songChanged)
{
    if (enabled) {
        DBUG << songChanged << lastState << MPDStatus::self()->state();
        bool stateChange=songChanged || lastState!=MPDStatus::self()->state();
        bool isRepeat=!stateChange && MPDState_Playing==MPDStatus::self()->state() &&
                      MPDStatus::self()->timeElapsed()>=0 && MPDStatus::self()->timeElapsed()<2;
        if (!stateChange && !isRepeat) {
            return;
        }
        lastState=MPDStatus::self()->state();
        switch (lastState) {
        case MPDState_Paused:
            commitTimer->pause();
            break;
        case MPDState_Playing: {
            time_t now=time(NULL);
            DBUG << "commitCurrent:" << commitedCurrent << "now:" << now << "isRepeat:" << isRepeat;

            if (isRepeat) {
                calcCommitTime();
                commitedCurrent=false;
                commitTimer->start();
                return;
            }
            if (!commitedCurrent) {
                commitTimer->start();
            }
            break;
        }
        default:
            commitTimer->stop();
        }
    }
}

void StatsDb::mpdStatusUpdated(const MPDStatusValues &vals)
{
    if (!vals.playlistLength) {
        currentSong.clear();
        commitTimer->stop();
    }
}
